<?php
/*
 * LanguageBehavior class file - 1.0.6
 *
 * @Backslider, Matti Ressler<software@suomedia.com>
 *
 *
 * Copyright  2012 by Suomedia - http://www.suomedia.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 *     Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *     Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer 
 * 	   in the documentation and/or other materials provided with the distribution.
 *     Neither the name of Suomedia nor the names of its contributors may be used to endorse or promote products derived from this software 
 * 	   without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
class LanguageBehavior extends CActiveRecordBehavior
{
	/* @var string the name of the translation model class. */
	public $translationClass;
	
	/* @var string the foreign key of the translation database table. */
	public $translationForeignKey;
	
	/* @var string the language id column of the translation database table, eg. 'language_id' */
	public $languageColumn;
	
	/* @var string the name of our language relation */
	public $languageRelation;
	
	/* @var array the translated attributes of the translation database table. */
	public $translationColumns;
	
	/* @var array the application languages */
	public $languages;
	
	public function attach($owner) 
	{		
		parent::attach($owner);
		
		// for front end selected language
		if(isset(Yii::app()->session['language_id'])) {
			$class = CActiveRecord::HAS_ONE;
			$owner->getMetaData()->relations[$this->languageRelation] = new $class($this->languageRelation, $this->translationClass, $this->translationForeignKey, array('condition' => $this->languageColumn . ' = ' . Yii::app()->session['language_id']));
		}
	}
	
	public function afterFind($event) 
	{
		// for back end - all translations
		if(!isset(Yii::app()->session['language_id']))
		{
			// fetch our translation data
			$owner = $this->getOwner();
			$owner_pk = $owner->getPrimaryKey();
			$model = call_user_func(array($this->translationClass, 'model'));
			$criteria = new CdbCriteria();
			$criteria->params = array('id'=>$owner_pk);
			$criteria->condition = "{$this->translationForeignKey}=:id";
			$criteria->index = $this->languageColumn;
			$translation = $model->findAll($criteria);
			foreach ($this->languages as $val) {
				foreach ($this->translationColumns as $field) {
					if(isset($translation[$val['id']])) {
						$owner->{$field}[$val['id']] = $translation[$val['id']]->$field;
					} else {
						$owner->{$field}[$val['id']] = null;
					}
				}
			}
		}
	}
	
	public function afterValidate($event)
	{
		$owner = $this->getOwner();
		$owner_class = get_class($owner);
		
		// grab our posted translation attributes
		foreach ($this->translationColumns as $field) {
			if(isset($_POST[$owner_class][$field])) 
				$owner->$field = $_POST[$owner_class][$field];
		}
		
		// validate translation attributes
		foreach ($this->languages as $val) {
			$translation = new $this->translationClass;
			foreach ($this->translationColumns as $field) {
				$translation->$field = $owner->{$field}[$val['id']];
				if(!$translation->validate(array($field))) {
					$owner->translation_error[$field][$val['id']] = $val['name'] . ' ' . $translation->getError($field);
				}
			}
		}
		// add errors to model errors for display at top of form
		$owner->addErrors($owner->translation_error);
	}

	public function afterSave($event) 
	{
		$owner = $this->getOwner();
		$owner_pk = $owner->getPrimaryKey();

		if (!$owner->isNewRecord) {
			$model = call_user_func(array($this->translationClass, 'model'));
			$criteria = new CdbCriteria();
			$criteria->params = array('id'=>$owner_pk);
			$criteria->condition = "{$this->translationForeignKey}=:id";
			$criteria->index = $this->languageColumn;
			$translations = $model->findAll($criteria);              
		}
		foreach ($this->languages as $val) {
			if (!isset($translations[$val['id']])) {
				$translation = new $this->translationClass;          
				$translation->{$this->languageColumn} = $val['id'];
				$translation->{$this->translationForeignKey} = $owner_pk;          
			} else {
				$translation = $translations[$val['id']];
			}
			foreach ($this->translationColumns as $field) {
				$translation->$field = $owner->{$field}[$val['id']];
			}
			$translation->save(false);
		}
	}
}	
